#!/usr/bin/ksh

_tmp_file=/tmp/space.$$.lst
_tmp_file2=/tmp/space2.$$.lst
_tmp_file3=/tmp/space3.$$.lst
_tmp_file4=/tmp/space4.$$.lst

__tmp_file=/tmp/space_.$$.lst

_sid=$1
_taux=75

## --
## -- Function detail_tbs
## --
function detail_tbs
{
typeset _tbs=$1
sqlplus -s /nolog <<! > /dev/null
connect / as sysdba
set head off
set feedback off
set trimspool on
set pagesize 0
set linesize 1000
spool $__tmp_file
select file_name, bytes / 1024 /1024 from dba_data_files where tablespace_name = '$_tbs';
!

cat $__tmp_file | egrep -v '^$' | while read file taille
do
    if [[ $(uname) = "SunOS" ]]; then
        typeset _fs=$(df -n $file | awk '{print $1}' | sed -e 's/://')
    elif [[ $(uname) = "HP-UX" ]]; then
        typeset _fs=$(df -n $file | awk '{print $1}' | sed -e 's/://')
    elif [[ $(uname) = "AIX" ]]; then
        typeset _fs=$(df -M $file | tail -1 | awk '{print $2}')
    fi

    print "$_tbs $_fs $file $taille"
done
}

## --
## -- Function detail_fs
## --
function detail_fs
{
typeset _fs=$1
if [[ $(uname) = "HP-UX" ]]; then
    typeset -i _fs_free_ko=$(df -k $_fs  | egrep 'free allocated Kb$' | awk '{print $1}')
    typeset -i _fs_free_mo=$((_fs_free_ko / 1024))
    typeset _vg=$(df $_fs | awk '{print $2}' | awk -F'/' '{print $3}')

elif [[ $(uname) = "AIX" ]]; then
    typeset -i _fs_free_ko=$(df -k $_fs  | tail -1 | awk '{print $3}')
    typeset -i _fs_free_mo=$((_fs_free_ko / 1024))

    typeset _lv=$(df -k $_fs  | tail -1 | awk '{print $1}' | awk -F'/' '{print $3}')
    typeset _vg=$(lslv $_lv | head -1 | awk '{print $NF}')
elif [[ $(uname) = "SunOS" ]]; then
    typeset -i _fs_free_ko=$(df -k $_fs  | tail -1 | awk '{print $4}')
    typeset -i _fs_free_mo=$((_fs_free_ko / 1024))

    typeset _vg=$(df -k $_fs | tail -1 | awk '{print $1}' | awk -F'/' '{print $5}')
fi
print "$_vg $_fs ${_fs_free_mo}"
}

## --
## -- Function detail_vg
## --
function detail_vg
{
typeset _vg=$1
if [[ $(uname) = "HP-UX" ]]; then
    typeset -i _vg_free_pe=$(/etc/vgdisplay $_vg | grep 'Free PE' | awk '{print $3}')
    typeset -i _vg_pe_size=$(/etc/vgdisplay $_vg | grep 'PE Size' | awk '{print $4}')
    typeset -i _vg_free_mo=$((_vg_free_pe * _vg_pe_size))

elif [[ $(uname) = "AIX" ]]; then
    typeset -i _vg_free_mo=$(lsvg $_vg | grep 'FREE PPs' | awk '{print $7}' | awk -F'(' '{print $2}')

elif [[ $(uname) = "SunOS" ]]; then
    typeset _vg_free_mo="?"
fi

print "$_vg $_vg_free_mo"
}

## --
## -- Chargement de l'environnement (interactif si $_sid est vide)
## --
. /product/oracle/admin/etc/base.ksh $_sid

if [[ -z $_sid ]]; then
    _sid=$ORACLE_SID
fi

## --
## -- Vision globale sur l'instance
## --

print "Instance : $_sid"
print "Serveur  : $(uname -a)"

# - Version de l'instance
sqlplus -s /nolog <<! > /dev/null
connect / as sysdba
set head off
set feedback off
set trimspool on
set pagesize 0
set linesize 1000
spool $_tmp_file
select banner from v\$version where rownum = 1;
exit;
!

print "Version  : $(cat $_tmp_file)"

## --
## -- Liste des tablespaces avec un taux d'occupation > $_taux
## --

print
print "## Liste des tablespaces avec un taux d'occupation > $_taux%"

sqlplus -s /nolog <<! > /dev/null
connect / as sysdba
spool $_tmp_file
@tbs.sql $_taux
exit;
!

cat $_tmp_file

## --
## -- Affichage des datafiles associes aux tablespaces problematiques
## --
print
print "## Liste des datafiles associes aux tablespaces problematiques"

for tbs in $(cat $_tmp_file | egrep "^$_sid " | awk '{print $4}')
do
    detail_tbs $tbs
done | sort > $_tmp_file2

typeset -i fslen=$(cat $_tmp_file2 | awk '{print length($2)}' | sort -n | tail -1)
typeset -i tbslen=$(cat $_tmp_file2 | awk '{print length($1)}' | sort -n | tail -1)

oldtbs=""
oldfs=""
while read tbs fs file taille
do
    if [[ -h $file ]]; then
        _cible=$(ls -l $file | awk '{print $NF}')
        if [[ -c $_cible ]]; then
            warning="(lien rawdevice)"
        else
            warning="(lien)"
        fi
    elif [[ -c $file ]]; then
        warning="(rawdevice)"
    else
        warning=""
    fi
    if [[ $oldtbs != $tbs ]]; then
        printf "%${tbslen}s %${fslen}s %5dMo $warning %s\n" $tbs $fs $taille $file
        oldtbs=$tbs
        oldfs=$fs
    else
        if [[ $oldfs != $fs ]]; then
            printf "%${tbslen}s %${fslen}s %5dMo $warning %s\n" "" $fs $taille $file
            oldfs=$fs
        else
            printf "%${tbslen}s %${fslen}s %5dMo $warning %s\n" "" "" $taille $file
        fi
    fi
done < $_tmp_file2

## --
## -- Affichage des filesystems associes aux tablespaces problematiques
## --
print
print "## Liste des filesystems associes aux tablespaces problematiques avec l'espace libre"

for fs in $(cat $_tmp_file2 | awk '{print $2}' | sort -u)
do
    detail_fs $fs
done | sort > $_tmp_file3

typeset -i vglen=$(cat $_tmp_file3  | awk '{print length($1)}' | sort -n | tail -1)
typeset -i fslen=$(cat $_tmp_file3  | awk '{print length($2)}' | sort -n | tail -1)

oldvg=""
while read vg fs free
do
    if [[ $oldvg != $vg ]]; then
        printf "%${vglen}s %${fslen}s %7dMo\n" $vg $fs $free
        oldvg=$vg
    else
        printf "%${vglen}s %${fslen}s %7dMo\n" "" $fs $free
    fi
done < $_tmp_file3

exit

## --
## -- Affichage des volume group associes aux tablespaces problematiques
## --
print
print "## Liste des volume group associes aux tablespaces problematiques avec l'espace libre"

for vg in $(cat $_tmp_file3 | awk '{print $1}' | sort -u)
do
    detail_vg $vg
done | sort > $_tmp_file4

typeset -i vglen=$(cat $_tmp_file4 | awk '{print length($1)}' | sort -n | tail -1)

while read vg free
do
    if [[ $free = "?" ]]; then
        printf "%${vglen}s       ?Mo\n" $vg
    else
        printf "%${vglen}s %7dMo\n" $vg $free
    fi
done < $_tmp_file4
